import requests
from urllib.parse import urlencode
from .config import AUTH_URL, TOKEN_URL
from .token_manager import read_tokens, write_tokens, is_token_valid

requests.packages.urllib3.disable_warnings()


class AllegroAuth:
    def __init__(self, client_id: str, client_secret: str, redirect_uri: str):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri

    def __send_request(self, method: str, data: dict):
        try:
            response = requests.request(
                method=method,
                url=TOKEN_URL,
                data=data,
                auth=(self.client_id, self.client_secret),
                verify=True,
                allow_redirects=False
            )
            response.raise_for_status()
            return response.json()
        except requests.RequestException as e:
            raise SystemExit(f"Request failed: {e}")

    def __get_authorization_code(self):
        params = urlencode({
            "response_type": "code",
            "client_id": self.client_id,
            "redirect_uri": self.redirect_uri
        })
        print(f"Open this URL and authorize the app:\n\n{AUTH_URL}?{params}\n")
        return input("Enter the code from URL: ")

    def __get_token_by_code(self, code: str):
        return self.__send_request(
            "POST",
            {
                "grant_type": "authorization_code",
                "code": code,
                "redirect_uri": self.redirect_uri
            })

    def __refresh_token(self, refresh_token: str):
        return self.__send_request(
            "POST",
            {
                "grant_type": "refresh_token",
                "refresh_token": refresh_token,
                "redirect_uri": self.redirect_uri
            })

    def authenticate(self) -> str:
        """ Authenticate with Allegro API and return access token."""
        
        token_data = read_tokens()

        if token_data and is_token_valid(token_data.expires_in, token_data.date):
            return token_data.access_token

        if token_data:
            new_tokens = self.__refresh_token(token_data.refresh_token)
        else:
            code = self.__get_authorization_code()
            new_tokens = self.__get_token_by_code(code)

        write_tokens(new_tokens)
        return new_tokens["access_token"]



import json
from datetime import datetime, timedelta
from unittest.mock import patch, mock_open
from allegro_auth.token_manager import (
    is_token_valid,
    read_tokens,
    write_tokens
)
from allegro_auth.schema import TokenSchema


def test_is_token_valid_true():
    now = datetime.now()
    date = (now - timedelta(seconds=10)).isoformat()
    assert is_token_valid(3600, date) is True

def test_is_token_valid_false():
    past = datetime.now() - timedelta(hours=2)
    date = past.isoformat()
    assert is_token_valid(3600, date) is False


@patch("builtins.open", new_callable=mock_open, read_data=json.dumps({
    "access_token": "abc",
    "token_type": "bearer",
    "refresh_token": "xyz",
    "expires_in": 3600,
    "scope": "allegro:all",
    "allegro_api": True,
    "iss": "allegro",
    "jti": "token-id",
    "date": datetime.now().isoformat()
}))
@patch("allegro_auth.token_manager.TOKEN_FILE", "dummy_path.json")
def test_read_tokens_valid(mock_file):
    token = read_tokens()
    assert isinstance(token, TokenSchema)
    assert token.access_token == "abc"


@patch("builtins.open", side_effect=FileNotFoundError)
@patch("allegro_auth.token_manager.TOKEN_FILE", "dummy_path.json")
def test_read_tokens_file_not_found(mock_file):
    assert read_tokens() is None


@patch("builtins.open", new_callable=mock_open)
@patch("allegro_auth.token_manager.TOKEN_FILE", "dummy_path.json")
@patch("allegro_auth.token_manager.datetime")
def test_write_tokens(mock_datetime, mock_file):
    mock_datetime.now.return_value = datetime(2025, 1, 1, 12, 0, 0)
    
    tokens = {
        "access_token": "abc",
        "token_type": "bearer",
        "refresh_token": "xyz",
        "expires_in": 3600,
        "scope": "allegro:all",
        "allegro_api": True,
        "iss": "allegro",
        "jti": "token-id"
    }

    write_tokens(tokens)

    written_data = "".join(call.args[0] for call in mock_file().write.call_args_list)
    result = json.loads(written_data)

    assert result["access_token"] == "abc"
    assert result["date"] == "2025-01-01T12:00:00"




from pydantic import BaseModel

class TokenSchema(BaseModel):
    access_token: str
    token_type: str
    refresh_token: str
    expires_in: int
    scope: str
    allegro_api: bool | None = None
    iss: str
    jti: str
    date: str


from platformdirs import user_config_dir
from pathlib import Path


AUTH_URL = "https://allegro.pl/auth/oauth/authorize"
TOKEN_URL = "https://allegro.pl/auth/oauth/token"
APP_NAME = "allegro-auth"
TOKEN_FILE = Path(user_config_dir(APP_NAME)) / "tokens.json"

# Ensure the folder exists
TOKEN_FILE.parent.mkdir(parents=True, exist_ok=True)


import json
from datetime import datetime, timedelta
from typing import Optional
from .schema import TokenSchema
from .config import TOKEN_FILE

def is_token_valid(expires_in: int, date: str) -> bool:
    token_time = datetime.fromisoformat(date)
    return (token_time + timedelta(seconds=expires_in)) > datetime.now()

def read_tokens() -> Optional[TokenSchema]:
    try:
        with open(TOKEN_FILE, 'r') as f:
            data = json.load(f)
            return TokenSchema(**data)
    except (json.JSONDecodeError, FileNotFoundError, ValueError) as e:
        print(e)
        return None

def write_tokens(tokens: dict):
    tokens['date'] = datetime.now().isoformat()
    with open(TOKEN_FILE, 'w') as f:
        json.dump(tokens, f, indent=2)


import pytest
from unittest.mock import patch, MagicMock
from allegro_auth.client import AllegroAuth


@pytest.fixture
def auth():
    return AllegroAuth("client_id", "client_secret", "http://localhost")


@patch("allegro_auth.client.read_tokens")
@patch("allegro_auth.client.is_token_valid", return_value=True)
def test_authenticate_returns_token(mock_valid, mock_read, auth):
    mock_token = MagicMock()
    mock_token.access_token = "valid_token"
    mock_token.expires_in = 3600
    mock_token.date = "2025-01-01T00:00:00"
    mock_read.return_value = mock_token

    token = auth.authenticate()
    assert token == "valid_token"


@patch("allegro_auth.client.read_tokens")
@patch("allegro_auth.client.is_token_valid", return_value=False)
@patch("allegro_auth.client.AllegroAuth._AllegroAuth__refresh_token")
@patch("allegro_auth.client.write_tokens")
def test_authenticate_refresh_token(mock_write, mock_refresh, mock_valid, mock_read, auth):
    mock_read.return_value = MagicMock(refresh_token="test_refresh")
    mock_refresh.return_value = {"access_token": "refreshed_token"}

    token = auth.authenticate()
    assert token == "refreshed_token"


@patch("allegro_auth.client.read_tokens", return_value=None)
@patch("allegro_auth.client.AllegroAuth._AllegroAuth__get_authorization_code", return_value="abc123")
@patch("allegro_auth.client.AllegroAuth._AllegroAuth__get_token_by_code", return_value={"access_token": "new_token"})
@patch("allegro_auth.client.write_tokens")
def test_authenticate_get_code(mock_write, mock_get_token, mock_get_code, mock_read, auth):
    token = auth.authenticate()
    assert token == "new_token"
